#!/usr/bin/env python
"""The network code for the client."""

# COPYRIGHT (C) 2003 Zachary White <zwhite@darkstar.frop.org>
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and
# that both that copyright notice and this permission notice appear in
# supporting documentation.
# 
# THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND 
# FITNESS, IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import socket, string, types, urllib

# A list of numerics that expect more than a single line response.
mline_start = ['203', '205', '212']
mline_end = ['204', '206', '213']

# Exceptions and the Error Codes that trigger them.
UnknownCommand = '400'
RedundantCommand = '401'
IllegalRequest = '402'

crlf = '\r\n'

class MusicClient:
	"""A class to handle communication with the music server."""

	def __init__(self, host='127.0.0.1', port=2323):
		self.music_socket = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
		self.music_socket.connect((host,port))
		self.file = self.music_socket.makefile('r')
		self.welcome = self.getline()

	def getline(self):
		"""Return one line from the server, stripping CRLF. Borrowed
		from ftplib.py"""
		line = self.file.readline()
		if not line: raise EOFError
		if line[-2:] == crlf:
			line = line[:-2]
		elif line[-1:] in crlf:
			line = line[:-1]
		return line

	def getmultiline(self):
		"""Get an entire response from the server.""" 
		line = self.getline()
		if line[:3] in mline_start:
			data = ''
			while 1:
				nextline = self.getline()
				if nextline[:3] in mline_end:
					return (line,data)
				else:
					data += nextline + '\n'
		else:
			return line

	def command(self, command):
		self.music_socket.send(command + '\n')
		return self.getmultiline()

	def add(self, filelist):
		"""A command to add one or more files.

		filelist: a list containing the files to add. May optionally
		be a string with a single file."""

		return urllib.unquote_plus(self.command('add %s' % urllib.quote_plus(string.join(filelist), '/')))

	def play(self):
		"""Start playing music."""
		return self.command('play')

	def stop(self):
		"""Stop playing music."""
		return self.command('stop')

	def pause(self):
		"""Pause the current song."""
		return self.command('pause')

	def random(self):
		"""Randomize the playlist."""
		return self.command('randomize')

	def sort(self):
		"""Sort the playlist asciibetically."""
		return self.command('sort')

	def playlist(self):
		"""Get a copy of the playlist. Returns a list containing the
		files from the server."""
		response = self.command('playlist')
		if type(response) == types.StringType:
			raise 'Unknown Error', response
		return string.split(response[1], '\n')

	def next(self):
		"""Go to the next song in the playlist."""
		return self.command('next')

	def list(self, filelist):
		"""Get a file listing from the server. Returns a list
		of files.

		filelist: A list of files/directiories to list. May also be a 
		string with a single file/directory.

		When the server returns an error, an IllegalRequest exception
		will be thrown."""
		if type(filelist) is types.StringType:
			filelist = [filelist]
		response = []
		for file in filelist:
			response.append(self.command('list %s' % urllib.quote_plus(file, '/')))
		returnlist = []
		for entry in response:
			if type(entry) == types.StringType:
				raise IllegalRequest, response
			for file in string.split(urllib.unquote_plus(entry[1]), '\n'):
				returnlist.append(file)
		return returnlist

	def quit(self):
		"""Close the server connection."""
		quitmsg = self.command('quit')
		self.music_socket.close()
		return quitmsg

class UDPstatus:
	"""A class used to recieve UDP status updates."""
	port = 0

	def __init__(self, port):
		"""Start listening for UDP requests. Sets self.port."""
		

if __name__ == '__main__':
	foo=MusicClient()
	print foo.command('next')
